/*
 * Copyright (c) 2004 Rob Braun
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Rob Braun nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * 1-Oct-2004
 * DRI: Rob Braun <bbraun@synack.net>
 */

#include <stdlib.h>

static unsigned char table [] = {
'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r',
's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2',
'3', '4', '5', '6', '7', '8', '9', '+', '/' };

static unsigned char decfunc(unsigned char input)
{
    switch(input) {
    case 'A': return 0;
    case 'B': return 1;
    case 'C': return 2;
    case 'D': return 3;
    case 'E': return 4;
    case 'F': return 5;
    case 'G': return 6;
    case 'H': return 7;
    case 'I': return 8;
    case 'J': return 9;
    case 'K': return 10;
    case 'L': return 11;
    case 'M': return 12;
    case 'N': return 13;
    case 'O': return 14;
    case 'P': return 15;
    case 'Q': return 16;
    case 'R': return 17;
    case 'S': return 18;
    case 'T': return 19;
    case 'U': return 20;
    case 'V': return 21;
    case 'W': return 22;
    case 'X': return 23;
    case 'Y': return 24;
    case 'Z': return 25;
    case 'a': return 26;
    case 'b': return 27;
    case 'c': return 28;
    case 'd': return 29;
    case 'e': return 30;
    case 'f': return 31;
    case 'g': return 32;
    case 'h': return 33;
    case 'i': return 34;
    case 'j': return 35;
    case 'k': return 36;
    case 'l': return 37;
    case 'm': return 38;
    case 'n': return 39;
    case 'o': return 40;
    case 'p': return 41;
    case 'q': return 42;
    case 'r': return 43;
    case 's': return 44;
    case 't': return 45;
    case 'u': return 46;
    case 'v': return 47;
    case 'w': return 48;
    case 'x': return 49;
    case 'y': return 50;
    case 'z': return 51;
    case '0': return 52;
    case '1': return 53;
    case '2': return 54;
    case '3': return 55;
    case '4': return 56;
    case '5': return 57;
    case '6': return 58;
    case '7': return 59;
    case '8': return 60;
    case '9': return 61;
    case '+': return 62;
    case '/': return 63;
    }
    return 255;
}

unsigned char* xar_from_base64(const unsigned char* input, int len)
{
    unsigned char b6 = 0;
    /*UNUSED unsigned char tmp = 0;*/
    int i, count;
    unsigned char* output;

    output = malloc(len);
    if( !output ) return NULL;
    i = count = 0;
    while(1) {
        if( i >= len-1 ) {
            output[count++] = '\0';
            return output;
        }
        b6 = decfunc(input[i++]) << 2;
        if( i >= len-1  || input[i]=='=') {
            output[count++] = b6;
            output[count++] = '\0';
            return output;
        }
        b6 |= decfunc(input[i]) >> 4;
        output[count++] = b6;

        b6 = decfunc(input[i++]) << 4;
        if( i >= len-1  || input[i]=='=') {
            output[count++] = b6;
            output[count++] = '\0';
            return output;
        }
        b6 |= decfunc(input[i]) >> 2;
        output[count++] = b6;

        b6 = decfunc(input[i++]) << 6;
        if( i >= len-1 || input[i]=='=' ) {
            output[count++] = b6;
            output[count++] = '\0';
            return output;
        }
        b6 |= decfunc(input[i++]);
        output[count++] = b6;
    }

    return output;
}

unsigned char* xar_to_base64(const unsigned char* input, int len)
{
    unsigned char b6;
    /*UNUSED unsigned char tmp; */
    unsigned char count = 0;
    int i=0;
    unsigned char* output;
    int outsize = (((float)len)*4/3)+5;

    output = malloc(outsize);
    if( !output )
        return NULL;

    while(1) {
        if( i >= len ) {
            output[count++] = '\n';
            output[count++] = '\0';
            return output;
        }
        b6 = input[i];
        b6 >>= 2;
        output[count++] = table[b6];

        b6 = input[i++];
        b6 &= 0x03;
        b6 <<= 4;
        if( i >= len ) {
            output[count++] = table[b6];
            output[count++] = '=';
            output[count++] = '=';
            output[count++] = '\n';
            output[count++] = '\0';
            return output;
        }
        b6 |= input[i] >> 4;
        output[count++] = table[b6];

        b6 = input[i++] & 0x0F;
        b6 <<= 2;
        if( i >= len ) {
            output[count++] = table[b6];
            output[count++] = '=';
            output[count++] = '\n';
            output[count++] = '\0';
            return output;
        }
        b6 |= input[i] >> 6;
        output[count++] = table[b6];

        b6 = input[i++] & 0x3F;
        output[count++] = table[b6];
    }
}

#ifdef _BTEST_
int main(int argc, char* argv[]) {
    unsigned char* enc = benc(argv[1], strlen(argv[1]));
    printf("%s", enc);
    printf("%s\n", bdec(enc, strlen(enc)));
}
#endif
